/*

             **************************************************
             ****    VEGA - Connectivity Generator V1.2    ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************


*/

#include <math.h>
#include <stdio.h>
#include <string.h>

#include "globdef.h"
#include "globvar.h"

#define  CONNTOL          1.2
#define  DISTCUTOFF         9


/**** Set covalent radii ****/

float *AssignCovRad(register ATOMO *Atomo1, ULONG TotAtomi)
{
  register ATOMO     *Atomo2;
  register float    *CovRad;
  register ULONG    j, k;

  if (CovRad = (float *)Alloca(sizeof(float) * TotAtomi)) {
    Atomo2  = Atomo1;
    for(k = 0;k < TotAtomi;++k) {
      for(j = 0;*AtmTable[j].Atom.C;++j)
        if (AtmTable[j].Atom.S == Atomo2 -> Elem.S) {
          CovRad[k] = AtmTable[j].CovRad;
          break;
        }
      Atomo2 = Atomo2 -> Ptr;
    }
  }
  return CovRad;
}


/**** Set Van Der Waals radii ****/

float *AssignVdwRad(register ATOMO *Atm, ULONG TotAtm, float ProRad, float *MaxRad)
{
  BOOL                  WrtAtm;
  float                 *VdwRad;
  register float        *Rad;
  register ULONG        k;

  BOOL                  Err = FALSE;

  if (VdwRad = (float *)Alloca(TotAtm * sizeof(float))) {
    Rad     = VdwRad;
    *MaxRad = 0.0;
    while(Atm) {
      WrtAtm = FALSE;
      for(k = 0;*AtmTable[k].Atom.C;++k) {
      	if (AtmTable[k].Atom.S == Atm -> Elem.S) {
          *Rad  = AtmTable[k].VdwRad + ProRad;
          if (*Rad > *MaxRad) *MaxRad = *Rad;
          WrtAtm = TRUE;
          ++Rad;
          break;
      	}
      } /* End of element loop */
      if (!WrtAtm) {
        PrintAtmErr("  Unknown VDW radius", Atm);
        Err = TRUE;
      }
      Atm = Atm -> Ptr;
    } /* End of atom loop */
    if (Err) {
      FREE(VdwRad);
      VdwRad = NULL;
    }
  }

  return VdwRad;
}


/**** Calculate the connectivity ****/

BOOL Connect(register ATOMO *Atomo1, ULONG TotAtomi)
{
  register ATOMO     *Atomo2;
  register float    *CovRad;
  register float    Dist, Tmp;
  register ULONG    j, k;

  BOOL              Ret = TRUE;

  PrintProg("  Building connectivity");


  if (CovRad = (float *)AssignCovRad(Atomo1, TotAtomi)) {

    /**** Check the connectivity ****/

    for(k = 0;(k < (TotAtomi - 1)) && (Ret);++k) {
      if ((*Atomo1 -> Name.C == 'H') && (Atomo1 -> NSost)) Atomo1 = Atomo1 -> Ptr;
      else {
      	Atomo2  = Atomo1 -> Ptr;
      	for(j = k + 1;(j < TotAtomi) && (Ret);++j) {
          Tmp   = Atomo1 -> x - Atomo2 -> x;
          Dist  = Quad(Tmp);
          Tmp   = Atomo1 -> y - Atomo2 -> y;
          Dist += Quad(Tmp);
          Tmp   = Atomo1 -> z - Atomo2 -> z;
          Dist += Quad(Tmp);
          if (Dist <= DISTCUTOFF) {
            Tmp = (CovRad[k] + CovRad[j]) * CONNTOL;
            if (Dist <= Quad(Tmp)) {
            if ((Atomo1 -> NSost < MAXBOND) && (Atomo1 -> NSost < MAXBOND)) {
              Atomo1 -> Conn[Atomo1 -> NSost] = Atomo2;
              Atomo2 -> Conn[Atomo2 -> NSost] = Atomo1;
              ++Atomo1 -> NSost;
                ++Atomo2 -> NSost;
                ++TotBond;
              } else {
                Errore("Unacceptable bond order");
                Ret = FALSE;
              }
            }
          }
          Atomo2 = Atomo2 -> Ptr;
      	}
      	Atomo1 = Atomo1 -> Ptr;
      }
    }
    FREE(CovRad);
  } else {
    PDBFree(Atomo1);
    return FALSE;
  }

  return Ret;
}

/**** Normalize the coordinates ****/

void Normalize(ATOMO *Top, ULONG TotAtomi)
{
  register float	MedX   = 0;
  register float	MedY   = 0;
  register float	MedZ   = 0;
  register ATOMO		*Atomo = Top;

  PrintProg("* Normalizing coordinates");

  do {
    MedX += Atomo -> x;
    MedY += Atomo -> y;
    MedZ += Atomo -> z;
    Atomo = Atomo -> Ptr;
  } while(Atomo);

  MedX /= TotAtomi;
  MedY /= TotAtomi;
  MedZ /= TotAtomi;
  Atomo = Top;

  do {
    Atomo -> x -= MedX;
    Atomo -> y -= MedY;
    Atomo -> z -= MedZ;
    Atomo       = Atomo -> Ptr;
  } while(Atomo);
}

