/*

             **************************************************
             ****            Global definitions            ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************


*/

#ifndef  __GLOBDEF
#define  __GLOBDEF

#define  VEGA_COPYRIGHT     "1996-2000"
#define  VEGA_DATE          "03.04.2000"
#define  VEGA_VER           "1.1"

#define  bittst(bit,num)    ((num) >> (bit) & 1)
#define  Cube(X)            ((X) * (X) * (X))
#define  Quad(X)            ((X) * (X))
#define  QuadDist(A,B)      (Quad((A) -> x - (B) -> x) + Quad((A) -> y - (B) -> y) + Quad((A) -> z - (B) -> z))
#define  Distance(A,B)      (SQR(QuadDist((A), (B))))

#define  CONV               (PI / 180.0)

#ifdef   INFINITE
#undef   INFINITE
#endif
#define  INFINITE           ((float)3.4e+38)

#define  RAD_TO_DEG         (180.0 / PI)
#define  DEG_TO_RAD         (PI / 180.0)
#define  LINELEN            90          /* I/O line buffer length I/O                */
#define  MAXBOND            6           /* Max bond order                            */
#define  MAXTRJATM          12          /* Max number of atom in trajectory analysis */
#define  PI                 M_PI
#define  DEBYE_CONST        4.8

/**** Atom flags ****/

#define  ATM_FLG_GENERIC    128         /* For generic use */

/**** Type of molecule ****/

#define  MOL_UNK            0          /* Unknown      */
#define  MOL_PROT           1          /* Protein      */
#define  MOL_NUCAC          2          /* Nucleic acid */

/**** Type of charge template ****/

#define  CHARGE_FRAG        0
#define  CHARGE_GAST        1

/**** Options for SubArg subroutine ****/

#define  SUBA_NOCHK         0
#define  SUBA_TOUP          1
#define  SUBA_POSFLOAT      2
#define  SUBA_POSINT        3


/****        Global switches        ****/
/**** Please don't change the order ****/

#define  GLOBSW_IFFIIUB         GlobSwitch[0]    /* Saves the IFF IIUB chunk    */
#define  GLOBSW_CONNSAV         GlobSwitch[1]    /* Saves the connectivity      */
#define  GLOBSW_IFFCALC         GlobSwitch[2]    /* Saves the IFF CALC chunk    */
#define  GLOBSW_CONNCALC        GlobSwitch[3]    /* Calculates the connectivity */
#define  GLOBSW_STDOUT          GlobSwitch[4]    /* Sets the output to stdout   */
#define  GLOBSW_MESTRJ          GlobSwitch[5]    /* Make a mesure in trajectory */

/**** Parameters for VLOGP ****/

#define  VLOGP_DOT_DENSITY             12
#define  VLOGP_PROBE_RAD              1.5

/**** Windows 9x/NT definitions ****/

#ifdef __WIN32__
#ifndef WIN32
#define  WIN32
#endif
#endif

/**** Silicon Graphics definitions ****/

#ifdef sgi
#define  SQR                fsqrt
#else
#define  SQR                sqrt
#endif

/**** AmigaOS sefinitions ****/

#ifdef AMIGA
#include "exec/types.h"
#define  ALLOC(X)           AllocVec((X), MEMF_ANY | MEMF_CLEAR)
#define  FREE               FreeVec
#else
typedef unsigned short      BOOL;
typedef char                BYTE;
typedef unsigned char       UBYTE;
typedef int                 LONG;
typedef unsigned int        ULONG;
typedef short               WORD;
typedef unsigned short      UWORD;
#define  FALSE              0
#define  TRUE               1

#define  ALLOC(X)           calloc((X), 1)
#define  FREE               free
#endif

#ifdef WIN32
#include <malloc.h>
#define  BIG_ENDIAN
#define  strcasecmp         strcmpi
#define  strncasecmp        strnicmp
#define  C_PLUSMIN          "\361"
#define  C_ANGSTROM         "\217"
#define  C_QUAD             "\375"
#define  C_CUBE             "\374"
#else
#define  C_PLUSMIN          "\261"
#define  C_ANGSTROM         "\305"
#define  C_QUAD             "\262"
#define  C_CUBE             "\263"
#endif


typedef union {
  char      C[2];
  WORD      S;
} DCHAR;

typedef union {
  char      C[4];
  LONG      L;
} QCHAR;

/**** Plane coefficients ****/

typedef struct {
  float     a, b, c, d;
} PLANE;

/**** Double precision vector ****/

typedef struct {
  double    x, y, z;
} VECTYPE;

/**** Atom description ****/

typedef struct {
  ULONG         Num;            /* Atom number               */
  QCHAR         Name;           /* Atom name                 */
  QCHAR         ResName;        /* Residue name              */
  QCHAR         ResSeq;         /* Number of residue         */
} ATMDESC;

/**** ATOMO structure ****/

typedef struct __PDBAtm {
  struct __PDBAtm   *Ptr;           /* Address of next structure */
  ULONG             Num;            /* Atom number               */
  QCHAR             Name;           /* Atom name                 */
  DCHAR             Elem;           /* Element                   */
  QCHAR             ResName;        /* Residue name              */
  QCHAR             ResSeq;         /* Number of residue         */
  float             x, y, z;        /* X, Y, Z coodinates        */
  float             Charge;         /* Atomic charge             */
  QCHAR             Pot;            /* Atom type (potential)     */
  struct __PDBAtm   *Conn[MAXBOND]; /* Connectivity matrix       */
  UBYTE             Order[MAXBOND]; /* Bond order: 2,3,4,6       */
  BYTE              NSost;          /* Number of bonded atoms    */
  BYTE              Flags;          /* ATOMO/HETATM, TER          */
  WORD              Ring;           /* Ring indicator            */
  BYTE              ChainID;        /* Chain indicator           */
} ATOMO;

/**** Force field parameter structure ****/

typedef struct {
  char              *CPtr;
  char              *Beg;
  FILE              *FFIN;    /* File Handle                  */
  ULONG             Line;     /* Current template line number */
  ULONG             TotArgs;  /* Total number of arguments    */
  UWORD             Args;     /* Current argument             */
} FFPAR;

/**** Electronic descriptor ****/

typedef struct {
  DCHAR             Atom;     /* Element                */
  BYTE              NSost;    /* Number of substituents */
  BYTE              Npi;      /* Number of Pi electrons */
} HUCK;

/**** Force field descriptor for atomic types ****/

typedef struct {
  DCHAR             Atom;     /* Element                */
  BYTE              NSost;    /* Number of substituents */
  BYTE              Ring;     /* Type of ring           */
  BYTE              Arom;     /* The ring is aromatic ? */
  BYTE              NSub;     /* Sublevel number        */
} FFDESC;

/**** File format structure ****/

typedef struct {
  LONG   Id;       /* Identification code */
  char   *Com;     /* Command token       */
  char   *Ext;     /* File extension      */
  LONG   Flags;    /* Control flags       */
} FILEFORMAT;


/**** Used with loader subroutine ****/

typedef union {
  char        Line[LINELEN + 1];
  LONG        Hdr;
} RECORD;

/**** Internal coordinates ****/

typedef struct {
  ATOMO    *Ptr;
  int       na, nb, nc;
  float     r, w, t;
  int       pos[3];
} ATMINT;

/**** Atom table ****/

typedef struct {
  DCHAR   Atom;     /* Chemical name        */
  float   Mass;     /* Atomic mass          */
  float   CovRad;   /* Covalent radius      */
  float   VdwRad;   /* Van Der Waals radius */
} ATMTAB;

/**** Parameter for trajectory analysis ****/

typedef struct {
  char          **Arg;          /* Arg list pointer          */
  UWORD         Num;            /* Number of atoms/args      */
  UWORD         Com;            /* Command                   */
  char          *OutFile;       /* Ptr to output file name   */
  FILEFORMAT    *OutForm;       /* Output format             */
  ATMDESC       Atm[MAXTRJATM]; /* Atom descriptions         */
} TRJPAR;

/**** Header for DCD trajectory ****/

typedef struct {
  QCHAR         Hdr;            /* Header (CORD)                     */
  ULONG         TotFrames;      /* Number of frames                  */
  ULONG         StartTime;      /* Starting time of first frame (fs) */
  ULONG         FrameStep;      /* Frame step                        */
  ULONG         Res1[5];        /* Not used                          */
  ULONG         NFixed;         /* Number of fixed atoms             */
  ULONG         Res2;           /* Not used                          */
  ULONG         Crystal;        /* Crystal flag                      */
  ULONG         Res3[8];        /* Not used                          */
  ULONG         Version;        /* CHARMm version                    */
} DCDHDR;

/**** Header for Quanta MSF ****/

typedef struct {
  ULONG   ns, nr, na;
  char    version[10];
  char    header[200];
} MSFHDR;

/**** Generic XYZ vector (sngle precision) ****/

typedef struct {
  float   x, y, z;
} XYZ;

/**** Preferences ****/

typedef struct {
  char          MOPAC_DEF[80];  /* Mopac: default parameters              */
  WORD          MOPAC_CRG;      /* Mopac: total charge                    */
  WORD          MOPAC_MMOK;     /* Mopac: polipeptidic correction         */
  BOOL          MOPAC_FLAG;     /* Mopac: TRUE when manual parameters     */
  float         SAS_PROBERAD;   /* SAS: probe radius                      */
  LONG          SAS_POINTS;     /* SAS: points density                    */
  FILEFORMAT    *OUTFORMAT;     /* Number of output format                */
  ULONG         RENSTART;       /* Starting residue for renumber          */
  QCHAR         ENRESNAME;      /* Residue name for energy calculation    */
  QCHAR         ENRRESSEQ;      /* Residue number for energy calculation  */
  float         ENCUTOFF;       /* Cutoff distance for energy calculation */
  float         ENFILTER;       /* Filter for energy calculation          */
  float         ENDIEL;         /* Dielectric for energy calculation      */
  float         VOL_DEN;        /* Dot density for volume calclulation    */
  float         SOL_RAD;        /* Solvatation radius                     */
  WORD          SOL_SHAPE;      /* Shape (0=Cubic, 1=Sphere, 2=shell      */
  char          *DCD_FILE;      /* Name of trajectory file                */
} PREFS;

#endif

