/*

             **************************************************
             ****            VEGA - Surface V1.1           ****
             **** Copyright 1996-2000, Alessandro Pedretti ****
             **************************************************


Supported surfaces:
- Insight SAS
- Quanta LogP
- Quanta MEP
- Quanta SAS
- VRML SAS dotted
- VRML SAS solid

*/

#include <stdio.h>
#include <math.h>
#include <string.h>

#include "globdef.h"
#include "globvar.h"
#include "formats.h"

#define  SRFNAMELEN   100


       /**** Calculates the surface accesible to solvent (SAS) ****/

BOOL Surface(FILE *FH, char *File, ATOMO *InizAtm, ULONG TotAtm,
             ULONG Points, float ProRad, UWORD Form, float *Contrib)
{
  ATOMO                  *Atm, *Atm2;
  BOOL                  Chk, WrtAtm;
  char                  FileNam[SRFNAMELEN], *Ptr;
  float                 MaxRad;
  float                 Xy, X, Y, Z;
  float                 X2, Y2, Z2;
  float                 *VdwRad, *Rad, *Rad2;
  ULONG                 j, k, Nu, PosFor;
  XYZ                   *SfPoint, *SfUn;

  BOOL                  Ret      = TRUE;
  float                 XyMax    = -INFINITE;
  float                 XyMin    = INFINITE;
  float                 Xyp      = 0;
  float                 Xyn      = 0;

#ifdef BIG_ENDIAN
  static const ULONG    Uno      = 0x01000000;
#else
  static const ULONG    Uno      = 1;
#endif

  ULONG                 NMol     = 1;
  ULONG                 TotPts   = 0;

  PrintProg("* Building SAS surface");

  if (VdwRad = AssignVdwRad(InizAtm, TotAtm, ProRad, &MaxRad)) {
    if (SfUn = CalcSfUn(Points, MaxRad, &Nu)) {

      /**** Map name definition ****/

      Xy = 1;
      for(Ptr = File + strlen(File);Ptr != File; --Ptr)
        if ((*Ptr == ':') || (*Ptr == '/')) {
          ++Ptr;
          break;
        }
      strcpy(FileNam, Ptr);
      for(Ptr = FileNam + strlen(FileNam);Ptr != FileNam; --Ptr)
        if (*Ptr == '.') {
          *Ptr = 0;
          break;
        }
      switch(Form) {
      case SRF_QUANTA:
      case SRF_QUANTA_LOGP:
#ifndef VLOGP
       Ret = PrintVlogPErr();
       break;
#endif
      case SRF_QUANTA_MEP:
        strcat(FileNam, ".msf");
        k = strlen(FileNam);
        memset(FileNam + k, ' ', 100 - k);;
        if (Ret = ForWrite(FH, (void *)&Uno, sizeof(ULONG)))
          Ret = ForWrite(FH, FileNam, 100);
        break;
      case SRF_INSIGHT:
        if (fprintf(FH, "DOTS\n") >= 0)
          Upper(FileNam);
        else Ret = PrintDosErr();
        break;
      case SRF_VRML_SOLID:
      case SRF_VRML_POINTS:
        if (fprintf(FH, "#VRML V1.0 ascii\n" \
                        "Separator {\n" \
                        "  Material{ diffuseColor  1.0 0.0 0.0 }\n" \
                        "  Coordinate3 { point [\n") < 0)
          Ret = PrintDosErr();
        break;
      }

      if (Ret) {
        Rad2 = VdwRad;
        for(Atm = InizAtm;(Ret) && (Atm); Atm = Atm -> Ptr) {
          SfPoint = SfUn;
          j       = 0;
          switch(Form) {
          case SRF_QUANTA:
          case SRF_QUANTA_LOGP:
          case SRF_QUANTA_MEP:
            PosFor  = ftell(FH);
            if ((fwrite(&j, sizeof(ULONG), 1, FH) != 1) ||    /* Header of fortran record */
                (SwapWrite(FH, &Atm -> Num) != 1) ||          /* Number of current atom   */
                (SwapWrite(FH, &NMol) != 1) ||                /* Number of molecule       */
                (fwrite(&j, sizeof(ULONG), 1, FH) != 1))      /* Number of dots           */
              Ret = PrintDosErr();
            break;
          case SRF_INSIGHT:
            WrtAtm = FALSE;
            break;
          case SRF_VRML_POINTS:
            break;
          }
          for(k = 0;(Ret) && (k < Nu);++k) {
            X   = Atm -> x + SfPoint -> x * *Rad2;
            Y   = Atm -> y + SfPoint -> y * *Rad2;
            Z   = Atm -> z + SfPoint -> z * *Rad2;
            Rad = VdwRad;
            Chk = TRUE;
            for(Atm2 = InizAtm;(Chk) && (Atm2); Atm2 = Atm2 -> Ptr) {
              X2 = fabs(X - Atm2 -> x);
              if (X2 <= MaxRad) {
                Y2 = fabs(Y - Atm2 -> y);
                if (Y2 <= MaxRad) {
                  Z2 = fabs(Z - Atm2 -> z);
                  if (Z2 <= MaxRad) {
                    if ((Quad(X2) + Quad(Y2) + Quad(Z2)) < Quad(*Rad))
                      Chk = FALSE;
                  }
                }
              }
              ++Rad;
            } /* End of atom loop */

            /**** Write the dot coordinates and eventually the property ****/

            if (Chk)  {
              switch(Form) {
              case SRF_QUANTA_LOGP:
              case SRF_QUANTA_MEP:
              case SRF_QUANTA:
                switch(Form) {
#ifdef VLOGP
                case SRF_QUANTA_LOGP:
                  Xy = CalcSrfLogP(InizAtm, Contrib, X, Y, Z);
                  if (Xy > 0) Xyp += Xy;
                  else Xyn += Xy;
                  break;
#endif
                case SRF_QUANTA_MEP:
                  Xy = CalcSrfMep(InizAtm, X, Y, Z);
                  break;
                }
                if (Xy > XyMax) XyMax = Xy;
                if (Xy < XyMin) XyMin = Xy;
                if ((SwapWrite(FH, &X) == 1) &&
                    (SwapWrite(FH, &Y) == 1) &&
                    (SwapWrite(FH, &Z) == 1) &&
                    (SwapWrite(FH, &Xy) == 1))
                  ++j;
                 else Ret = PrintDosErr();
                break;
              case SRF_INSIGHT:
                if (!WrtAtm) {
                  WrtAtm = TRUE;
                  if(fprintf(FH, "ATOMO_REF %s:%d:%.4s %.6f %.6f %.6f\n",
                             FileNam, NMol, Atm -> Name.C, Atm -> x, Atm -> y,
                             Atm -> z) < 0)
                    Ret = PrintDosErr();
                }
                if ((Ret) && (fprintf(FH, "%.6f %.6f %.6f 120\n", X, Y, Z) < 0))
                  Ret = PrintDosErr();
                break;
              case SRF_VRML_SOLID:
              case SRF_VRML_POINTS:
                if (fprintf(FH, "    %8.3f %8.3f %8.3f,\n", X, Y, Z) >= 0)
                  ++TotPts;
                else Ret = PrintDosErr();
                break;
              }
            }
/*
            printf("%d %4.4s %2.2s %3.3s_%4.4s -> %f %f %f\n",
                   Atm -> Num, Atm -> Name.C, Atm -> Elem.C,
                   Atm -> ResName.C, Atm -> ResSeq.C, X, Y, Z);
*/

            ++SfPoint;
          } /* End of dot loop */

          /**** Tuning-Up the Fortran record ****/

          if ((Form == SRF_QUANTA) || (Form == SRF_QUANTA_LOGP) ||
              (Form == SRF_QUANTA_MEP)) {
            if (j) {
              fseek(FH, PosFor, SEEK_SET);
              k = j * sizeof(float) * 4 + sizeof(ULONG) * 3;
              if (SwapWrite(FH, &k) == 1) {
                fseek(FH, sizeof(ULONG) * 2, SEEK_CUR);
                if (SwapWrite(FH, &j) == 1) {
                  k = j * sizeof(float) * 4 + sizeof(ULONG) * 3;
                  fseek(FH, 0, SEEK_END);
                  if (SwapWrite(FH, &k) != 1)
                    Ret = PrintDosErr();
                } else Ret = PrintDosErr();
              } else Ret = PrintDosErr();
            } else fseek(FH, PosFor, SEEK_SET);
          }
          ++Rad2;
          if (Atm -> Flags > 1) ++NMol;
        } /* End of atom loop */

        switch(Form) {
        case SRF_VRML_POINTS:
          if (fprintf(FH, "  ] }\n\n  PointSet {\n" \
                      "    startIndex 0\n    numPoints -1\n  }\n}\n") < 0)
            Ret = PrintDosErr();
          break;
        case SRF_VRML_SOLID:
          if (fprintf(FH, "  ] }\n\n  IndexedFaceSet {\n" \
                          "    coordIndex [\n      ") >= 0) {
            for(k = 0; (Ret) && (k < TotPts); ++k) {
              if (fprintf(FH, "%4d,", k) >= 0)
                if (!(k % 5))
                  if (fprintf(FH, "\n    ") < 0)
                    Ret = PrintDosErr();
              else Ret = PrintDosErr();
            }
            if ((Ret) && (fprintf(FH, "      -1 ]\n    }\n}\n") < 0))
              Ret = PrintDosErr();
          } else Ret = PrintDosErr();
          break;
#ifdef VLOGP
        case SRF_QUANTA_LOGP:
          PrintLogP(Xyp, Xyn);
#endif
        case SRF_QUANTA_MEP:
          if (!GlobSwitch[4])
            printf("  Range of values: %.4f -> %.4f\n", XyMin, XyMax);
          break;
        default:
          Ret = TRUE;
        }
      }
      FREE(SfUn);
    } else Ret = FALSE;
    FREE(VdwRad);
  } else Ret = FALSE;

  return Ret;
}


/**** Dot calculation of a sphere with radius = 1 ****/

XYZ *CalcSfUn(ULONG Points, float MaxRad, ULONG *Nu)
{
  double                Fi, Fj, Nequat, Z;
  register double       Xy;
  register XYZ          *SfPoint;
  ULONG                 j, k, Nhor, Nvert;
  XYZ                   *SfUn;

  Points *= (ULONG)((double)4.0 * (double)PI * (double)Quad(MaxRad));
  if (SfUn = (XYZ *)Alloca(Points * sizeof(XYZ))) {
    Nequat = (double)sqrt(PI * Points);  /* Linear density                 */
                                         /* Nequat = 2 * PI * SQR(D) * RAD */
    Nvert   = (ULONG)(Nequat / 2.0);
    *Nu     = 0;
    SfPoint = SfUn;
    for(k = 0; (k <= (ULONG)Nvert) && (*Nu < Points); ++k) {
      Fi   = (double)(PI * (double)k) / (double)Nvert;
      Z    = (double)cos(Fi);
      Xy   = (double)sin(Fi);
      Nhor = (ULONG)(Xy * Nequat);
      if (Nhor < 1) Nhor = 1;

      for(j = 0;(j <= (ULONG)(Nhor - 1)) && (*Nu < Points); ++j) {
        Fj = (2 * PI * (double)j) / (double)Nhor;
        SfPoint -> x = (float)((double)cos(Fj) * (double)Xy);
        SfPoint -> y = (float)((double)sin(Fj) * (double)Xy);
        SfPoint -> z = (float)Z;
        ++*Nu;
        ++SfPoint;
      } /* End of loop */
    } /* End of loop */
  }

  return SfUn;
}


/**** Calculate the surface area ****/

float CalcSrfArea(ATOMO *InizAtm, float *InizRad, XYZ *InizSfUn,
                  ULONG Nu, TRJPAR *Par, float MaxRad)
{
  BOOL          Chk;
  ATOMO         *Atm1, *Atm2;
  float         *Rad1, *Rad2;
  float         Xp, Yp, Zp;
  ULONG         SfDots;
  ULONG         j, k;
  XYZ           *SfUn;

  float         Area = 0.0;

  Rad1 = InizRad;
  for(Atm1 = InizAtm; (Atm1); Atm1 = Atm1 -> Ptr) {
    SfDots = 0;
    SfUn   = InizSfUn;
    if ((Par) && (Par -> Num)) {
      for(j = 0; (Atm1 -> Num != Par -> Atm[j].Num) && (j < (ULONG)Par -> Num); ++ j);
      if (j == (ULONG)Par -> Num) {
        ++Rad1;
        continue;
      }
    }

    for(k = 0; k < Nu; ++k) {
      Rad2 = InizRad;
      Chk  = TRUE;
      Xp   = (SfUn -> x * *Rad1) + Atm1 -> x;
      Yp   = (SfUn -> y * *Rad1) + Atm1 -> y;
      Zp   = (SfUn -> z * *Rad1) + Atm1 -> z;
      for(Atm2 = InizAtm; (Atm2) && (Chk); Atm2 = Atm2 -> Ptr) {
        if (Atm1 != Atm2) {
          if ((Quad(Atm2 -> x - Xp) +
               Quad(Atm2 -> y - Yp) +
               Quad(Atm2 -> z - Zp)) < Quad(*Rad2))

            Chk = FALSE;
        }
        ++Rad2;
      } /* End of atom loop */
      if (Chk) ++SfDots;
      ++SfUn;
    }  /* End of dot loop */
    Area += (4.0 * PI * Quad(*Rad1) / Nu) * SfDots;
    ++Rad1;
  } /* End of atom loop */

  return (float)Area;
}
